@@ -641,6 +641,8 @@ int cw_gen_stop_internal(cw_gen_t *gen)
 		return CW_FAILURE;
 	}
 
+	cw_debug_msg ((&cw_debug_object_dev), CW_DEBUG_GENERATOR, CW_DEBUG_INFO,
+		      "libcw/gen: gen->do_dequeue_and_play = false");
 	gen->do_dequeue_and_play = false;
 
 	if (!gen->thread.running) {
@@ -652,7 +654,11 @@ int cw_gen_stop_internal(cw_gen_t *gen)
 		return CW_SUCCESS;
 	}
 
-	/* This is to wake up cw_signal_wait_internal() function that
+	/* "while (gen->do_dequeue_and_play)" loop in thread function
+	   may be in a state where dequeue() function returned IDLE
+	   state, and the loop is waiting for new tone.
+
+	   This is to wake up cw_signal_wait_internal() function that
 	   may be waiting idle for signal in "while ()" loop in thread
 	   function. */
 	pthread_kill(gen->thread.id, SIGALRM);
@@ -682,7 +688,8 @@ int cw_gen_stop_internal(cw_gen_t *gen)
 	*/
 
 
-#if 0 /* Old code using pthread_kill() instead of pthread_join(). */
+#if 0 /* Old code using pthread_kill() instead of pthread_join().
+	 This code is unused since before 2015-08-30. */
 
 	struct timespec req = { .tv_sec = 1, .tv_nsec = 0 };
 	cw_nanosleep_internal(&req);
@@ -894,8 +901,10 @@ void *cw_gen_dequeue_and_play_internal(void *arg)
 		     queue is "low/critical"; client code then can add more
 		     characters to the queue; the observation is done using
 		     cw_wait_for_tone_queue_critical();
-		   - ...
 
+		   - allows client code to observe any dequeue event
+                     by waiting for signal in cw_wait_for_tone() /
+                     cw_tq_wait_for_tone_internal()
 		 */
 		pthread_kill(gen->client.thread_id, SIGALRM);
 
@@ -1977,18 +1986,58 @@ int cw_gen_play_eow_space_internal(cw_gen_t *gen)
 	   this in this function), the tone queue manager can
 	   recognize level dropping from 2 to 1. Then the passing of
 	   critical level can be noticed, and "low level" callback can
-	   be called. */
+	   be called.
+
+	   BUT: Sometimes the first tone is dequeued before/during the
+	   second one is enqueued, and we can't recognize 2->1 event.
+
+	   So, to be super-sure that there is a recognizable event of
+	   passing tone queue level from 2 to 1, we split the eow
+	   space into N parts and enqueue them. This way we have N + 1
+	   tones per space, and client applications that rely on low
+	   level threshold == 1 can correctly work when enqueueing
+	   spaces.
+
+	   At 60 wpm length of gen->eow_space_len is 100000 [us], so
+	   it's large enough to safely divide it by small integer
+	   value. */
+
+	int enqueued = 0;
 
 	cw_tone_t tone;
-	CW_TONE_INIT(&tone, 0, gen->eow_space_len, CW_SLOPE_MODE_NO_SLOPES);
-	int rv = cw_tq_enqueue_internal(gen->tq, &tone);
-
-	if (rv == CW_SUCCESS) {
-		CW_TONE_INIT(&tone, 0, gen->adjustment_space_len, CW_SLOPE_MODE_NO_SLOPES);
-		rv = cw_tq_enqueue_internal(gen->tq, &tone);
+#if 0
+	/* This section is incorrect. Enable this section only for
+	   tests.  This section "implements" a bug that was present in
+	   libcw until version 6.4.1 and that is now tested by
+	   src/libcw/tests/libcw_test_tq_short_space.c */
+	int n = 1; /* No division. Old situation causing an error in
+		      client applications. */
+#else
+	int n = 2; /* "small integer value" - used to have more tones per eow space. */
+#endif
+	CW_TONE_INIT(&tone, 0, gen->eow_space_len / n, CW_SLOPE_MODE_NO_SLOPES);
+	for (int i = 0; i < n; i++) {
+		int rv = cw_tq_enqueue_internal(gen->tq, &tone);
+		if (rv) {
+			enqueued++;
+		} else {
+			return CW_FAILURE;
+		}
 	}
 
-	return rv;
+	CW_TONE_INIT(&tone, 0, gen->adjustment_space_len, CW_SLOPE_MODE_NO_SLOPES);
+	int rv = cw_tq_enqueue_internal(gen->tq, &tone);
+	if (rv) {
+		enqueued++;
+	} else {
+		return CW_FAILURE;
+	}
+
+	cw_debug_msg ((&cw_debug_object), CW_DEBUG_GENERATOR, CW_DEBUG_DEBUG,
+		      "libcw: enqueued %d tones per eow space, tq len = %d",
+		      enqueued, cw_tq_length_internal(gen->tq));
+
+	return CW_SUCCESS;
 }
 
 
@@ -2536,9 +2585,8 @@ unsigned int test_cw_gen_new_delete_internal(void)
 	int n = 100;
 
 	/* new() + delete() */
-	fprintf(stderr, "libcw/gen: generator test 1/4\n");
 	for (int i = 0; i < n; i++) {
-
+		fprintf(stderr, "libcw/gen: generator test 1/4, loop #%d/%d\n", i, n);
 
 		cw_gen_t *gen = cw_gen_new_internal(CW_AUDIO_NULL, NULL);
 		cw_assert (gen, "failed to initialize generator (loop #%d)", i);
@@ -2590,8 +2638,7 @@ unsigned int test_cw_gen_new_delete_internal(void)
 	}
 
 
-
-	/* Inner loop. */
+	/* Inner loop limit. */
 	int m = n;
 
 
