@@ -700,8 +700,7 @@ int cw_tq_enqueue_internal(cw_tone_queue_t *tq, cw_tone_t *tone)
 	}
 
 
-	cw_debug_msg ((&cw_debug_object_dev), CW_DEBUG_TONE_QUEUE, CW_DEBUG_DEBUG,
-		      "libcw/tq: enqueue tone %d us, %d Hz", tone->len, tone->frequency);
+	// cw_debug_msg ((&cw_debug_object_dev), CW_DEBUG_TONE_QUEUE, CW_DEBUG_DEBUG, "libcw/tq: enqueue tone %d us, %d Hz", tone->len, tone->frequency);
 
 	/* Enqueue the new tone.
 
@@ -718,9 +717,9 @@ int cw_tq_enqueue_internal(cw_tone_queue_t *tq, cw_tone_t *tone)
 		/* A loop in cw_gen_dequeue_and_play_internal()
 		   function may await for the queue to be filled with
 		   new tones to dequeue and play.  It waits for a
-		   signal, for information that there are some new
-		   tones in tone queue. This is a right place and time
-		   to send such a signal. */
+		   notification from tq that there are some new tones
+		   in tone queue. This is a right place and time to
+		   send such notification. */
 		tq->state = CW_TQ_BUSY;
 		pthread_kill(tq->gen->thread.id, SIGALRM);
 	}
@@ -888,7 +887,7 @@ int cw_tq_wait_for_level_internal(cw_tone_queue_t *tq, uint32_t level)
 		return CW_FAILURE;
 	}
 
-	/* Wait until the queue length is at or below criticality. */
+	/* Wait until the queue length is at or below critical level. */
 	while (cw_tq_length_internal(tq) > level) {
 		cw_signal_wait_internal();
 	}
@@ -949,6 +948,7 @@ void cw_tq_flush_internal(cw_tone_queue_t *tq)
 	/* Empty and reset the queue. */
 	tq->len = 0;
 	tq->head = tq->tail;
+	tq->state = CW_TQ_IDLE;
 
 	pthread_mutex_unlock(&(tq->mutex));
 
@@ -1716,4 +1716,60 @@ unsigned int test_cw_tq_enqueue_internal_2(void)
 
 
 
+/*
+
+  tests::cw_tq_wait_for_level_internal()
+*/
+unsigned int test_cw_tq_wait_for_level_internal(void)
+{
+	int p = fprintf(stdout, "libcw/tq: testing correctness of waiting for level:");
+
+	cw_tone_t tone;
+	CW_TONE_INIT(&tone, 20, 10000, CW_SLOPE_MODE_STANDARD_SLOPES);
+
+	for (int i = 0; i < 10; i++) {
+		cw_gen_t *gen = cw_gen_new_internal(CW_AUDIO_NULL, CW_DEFAULT_NULL_DEVICE);
+		cw_assert (gen, "failed to create a tone queue\n");
+		cw_gen_start_internal(gen);
+
+		/* Test the function for very small values,
+		   but for a bit larger as well. */
+		int level = i <= 5 ? i : 10 * i;
+
+		/* Add a lot of tones to tone queue. "a lot" means three times more than a value of trigger level. */
+		for (int j = 0; j < 3 * level; j++) {
+			int rv = cw_tq_enqueue_internal(gen->tq, &tone);
+			cw_assert (rv, "failed to enqueue tone #%d", j);
+		}
+
+		int rv = cw_tq_wait_for_level_internal(gen->tq, level);
+		cw_assert (rv == CW_SUCCESS, "cw_tq_wait_for_level_internal() failed for level = %d", level);
+
+		size_t len = cw_tq_length_internal(gen->tq);
+
+		/* cw_tq_length_internal() is called after return of
+		   tested function, so 'len' can be smaller by one,
+		   but never larger, than 'level'.
+
+		   During initial tests, for function implemented with
+		   signals and with alternative IPC, diff was always
+		   zero on my primary Linux box. */
+		int diff = level - len;
+		cw_assert (abs(diff) <= 1, "difference is too large: level = %d, len = %zd, diff = %d\n", level, len, diff);
+
+		fprintf(stderr, "          level = %d, len = %zd, diff = %d\n", level, len, diff);
+
+		cw_gen_stop_internal(gen);
+		cw_gen_delete_internal(&gen);
+	}
+
+	CW_TEST_PRINT_TEST_RESULT (false, p);
+
+	return 0;
+}
+
+
+
+
+
 #endif /* #ifdef LIBCW_UNIT_TESTS */
